package cz.cvut.kbss.reporting.service.repository;

import cz.cvut.kbss.reporting.model.OccurrenceReport;
import cz.cvut.kbss.reporting.model.Tag;
import cz.cvut.kbss.reporting.persistence.dao.GenericDao;
import cz.cvut.kbss.reporting.persistence.dao.OccurrenceReportDao;
import cz.cvut.kbss.reporting.service.security.SecurityUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;
import java.util.List;
import java.util.Optional;

@Service
public class OccurrenceReportService extends AbstractRepositoryService<OccurrenceReport> {

    private final OccurrenceReportDao dao;

    private final SecurityUtils securityUtils;

    @Autowired
    public OccurrenceReportService(OccurrenceReportDao dao, SecurityUtils securityUtils) {
        this.dao = dao;
        this.securityUtils = securityUtils;
    }

    public List<OccurrenceReport> findAll(Tag tag) {
        return dao.findAll(tag);
    }


    @Override
    protected GenericDao<OccurrenceReport> getPrimaryDao() {
        return dao;
    }

    @Override
    void prePersist(OccurrenceReport instance) {
        persistTags(instance);
        instance.setAuthor(securityUtils.getCurrentUser());
        instance.setCreated(LocalDateTime.now());
    }

    @Override
    void preUpdate(OccurrenceReport instance) {
        persistTags(instance);
        instance.setLastEditor(securityUtils.getCurrentUser());
        instance.setLastModified(LocalDateTime.now());
    }

    private void persistTags(OccurrenceReport instance) {
        Optional.ofNullable(instance.getTags())
                .ifPresent(
                        tags -> tags.stream()
                                    .filter(t -> t.getAuthor() == null)
                                    .forEach(
                                            t -> {
                                                t.setCreated(LocalDateTime.now());
                                                t.setAuthor(securityUtils.getCurrentUser());
                                            }
                                    )
                );
    }
}
