'use strict';

import React from "react";
import Reflux from "reflux";
import {MenuItem, Nav, Navbar, NavBrand, NavDropdown, NavItem} from "react-bootstrap";
import {Route, Switch} from "react-router-dom";
import {LinkContainer} from "react-router-bootstrap";

import Actions from "../actions/Actions";
import Authentication from "../utils/Authentication";
import Constants from "../constants/Constants";
import DashboardController from "./dashboard/DashboardController";
import I18nWrapper from "../i18n/I18nWrapper";
import injectIntl from "../utils/injectIntl";
import Mask from "./Mask";
import ProfileController from "./profile/ProfileController";
import ReportController from "./report/ReportController";
import ReportsController from "./report/ReportsController";
import Routes from "../utils/Routes";
import UserStore from "../stores/UserStore";

class MainView extends Reflux.Component {
    constructor(props) {
        super(props);
        this.state = {showProfile: false};
        this.store = UserStore;
        this.i18n = props.i18n;
    }

    componentWillMount() {
        super.componentWillMount();
        Actions.loadUser();
    }

    openUserProfile = () => {
        this.setState({showProfile: true});
    };

    closeUserProfile = () => {
        this.setState({showProfile: false});
    };

    render() {
        if (!this.state.loaded) {
            return <Mask/>;
        }
        if (this.state.user === null) {
            return (<div>{this.props.children}</div>);
        }
        const name = this.state.user.firstName.substr(0, 1) + '. ' + this.state.user.lastName;
        return <div>
            <header>
                <Navbar fluid={true}>
                    <Navbar.Header>
                        <Navbar.Brand>{Constants.APP_NAME}</Navbar.Brand>
                    </Navbar.Header>
                    <Nav>
                        <LinkContainer
                            to={Routes.dashboard.path}><NavItem>{this.i18n('main.dashboard-nav')}</NavItem></LinkContainer>
                        <LinkContainer
                            to={Routes.reports.path}><NavItem>{this.i18n('main.reports-nav')}</NavItem></LinkContainer>
                    </Nav>
                    <Nav pullRight style={{margin: '0 -15px 0 0'}}>
                        <NavDropdown id='logout' title={name}>
                            <MenuItem onClick={this.openUserProfile}>{this.i18n('main.user-profile')}</MenuItem>
                            <MenuItem href='#' onClick={Authentication.logout}>{this.i18n('main.logout')}</MenuItem>
                        </NavDropdown>
                    </Nav>
                </Navbar>
            </header>
            <section style={{height: '100%'}}>
                <ProfileController show={this.state.showProfile} onClose={this.closeUserProfile}/>
                <Switch>
                    <Route path={Routes.dashboard.path} component={DashboardController}/>
                    <Route path={Routes.createReport.path} exact component={ReportController}/>
                    <Route path={Routes.editReport.path} component={ReportController}/>
                    <Route path={Routes.reports.path} component={ReportsController}/>
                </Switch>
            </section>
        </div>;
    }
}

export default injectIntl(I18nWrapper(MainView));
