import React from "react";
import Reflux from "reflux";
import PropTypes from "prop-types";
import {Button, Glyphicon, Label} from "react-bootstrap";
import Typeahead from "react-bootstrap-typeahead";

import Actions from "../../actions/Actions";
import I18nWrapper from "../../i18n/I18nWrapper";
import injectIntl from "../../utils/injectIntl";
import TagStore from "../../stores/TagStore";
import TypeaheadResultList from "../typeahead/TypeaheadResultList";

class Tags extends Reflux.Component {

    constructor(props) {
        super(props);
        this.i18n = props.i18n;
        this.state = {renderCreateButton: false};
        this.store = TagStore;
    }

    componentDidMount() {
        Actions.loadTags();
    }

    _onAddTag = (tag) => {
        const tags = this.props.tags;
        if (tags.find(t => t.name === tag.name)) {
            return; // Tag already present
        }
        const tagsCopy = tags.slice();
        tagsCopy.push(tag);
        this.props.onChange({tags: tagsCopy});
        this.typeahead.resetSelection();
        this.setState({renderCreateButton: false});
    };

    _onCreateTag = () => {
        const name = this.typeahead.getEntryText();
        if (this.state.tags.find(t => t.name === t)) {
            return; // Tag already exists
        }
        this._onAddTag({name: name});
        this.setState({renderCreateButton: false});
    };

    _onRemoveTag = (tag) => {
        const tagsCopy = this.props.tags.slice();
        tagsCopy.splice(tagsCopy.indexOf(tag), 1);
        this.props.onChange({tags: tagsCopy});
    };

    _onTagEntryChange = (e) => {
        if (e.target.value !== '') {
            this.setState({renderCreateButton: true});
        } else {
            this.setState({renderCreateButton: false});
        }
    };

    render() {
        return <div>
            <div><label className='control-label'>{this.i18n('tags.header')}</label></div>

            <div className='tags'>
                {this._renderTags()}
            </div>
            <div className='row'>
                <div className='col-xs-2'>
                    <Typeahead ref={c => this.typeahead = c} options={this.state.tags} onOptionSelected={this._onAddTag}
                               placeholder={this.i18n('tags.search.placeholder')} size='small'
                               filterOption='name' displayOption='name' formInputOption='id'
                               customListComponent={TypeaheadResultList} onChange={this._onTagEntryChange}/>
                </div>
                <div className='col-xs-1'>
                    {this._renderCreateTagButton()}
                </div>
            </div>
        </div>;
    }

    _renderTags() {
        const tagElements = [],
            tags = this.props.tags;
        for (let i = 0, len = tags.length; i < len; i++) {
            tagElements.push(<Label key={tags[i].name} bsStyle='info'>
                {tags[i].name}
                <Glyphicon glyph='remove' className='label-glyph' title={this.i18n('tags.remove.tooltip')}
                           onClick={() => this._onRemoveTag(tags[i])}/>
            </Label>);
        }
        return tagElements;
    }

    _renderCreateTagButton() {
        if (this.state.renderCreateButton) {
            return <Button title={this.i18n('tags.create.tooltip')} bsSize='small' bsStyle='success'
                           onClick={this._onCreateTag}>
                <Glyphicon glyph='plus'/>
            </Button>;
        }
        return null;
    }
}

Tags.propTypes = {
    tags: PropTypes.array,
    onChange: PropTypes.func.isRequired
};

Tags.defaultProps = {
    tags: []
};

export default injectIntl(I18nWrapper(Tags));
